/**
 * Ref Mode ["5", "4.096", "2.5"]
 * Lookup Table values for bypass scale, digital scale, fsr scale, vref max based on the ref mode
 */
let bypassScale = [1.5, 1.5, 1];
let digitalScale = [1.333333333, 1.092266667, 1];
let fsrScale = [1, 1.22070312, 2];
let vrefMax = [5.1, 4.2, 2.6];

/**
 * Function that needs to be exported for getting the LSB Size
 * arguments should be in the same order as it will be passed from the GUI
 * controlData   : Entire Object
 * parameters     : Parameters stored in the JSON file
 * common        : Common Controls in parameters JSON file
 * codeToVoltage : Code to Voltage Controls in parameters JSON file
 * voltageToCode : Voltage to Code Controls in parameters JSON file
 */

export function getLsbSize(
  controlData,
  parameters,
  common,
  codeToVoltage,
  voltageToCode,
) {
  return (
    getFsrValue(controlData, parameters, common, codeToVoltage, voltageToCode) /
    getNumOfCodesValue(parameters)
  );
}

/**
 * Function that needs to be exported for getting the FSR Value
 * arguments should be in the same order as it will be passed from the GUI
 * controlData   : Entire Object
 * parameters     : Parameters stored in the JSON file
 * common        : Common Controls in parameters JSON file
 * codeToVoltage : Code to Voltage Controls in parameters JSON file
 * voltageToCode : Voltage to Code Controls in parameters JSON file
 */

export function getFsrValue(
  controlData,
  parameters,
  common,
  codeToVoltage,
  voltageToCode,
) {
  let refMode = common.refMode.value;
  let refModeIndex = common.refMode.options.indexOf(`${refMode}`);
  let filterType = common.filter.value;
  let pgaGain = parseInt(common.pgaGain.value);
  let digitalScaleVal = digitalScale[refModeIndex];
  let bypassScaleVal = bypassScale[refModeIndex];
  let fsrScaleVal = fsrScale[refModeIndex];

  common.refVoltage.maxValue = vrefMax[refModeIndex];
  if (
    parseFloat(common.refVoltage.value) > parseFloat(common.refVoltage.maxValue)
  ) {
    common.refVoltage.value = parseFloat(common.refVoltage.maxValue);
  }
  if (common.pgaGain.value == "Bypass") {
    pgaGain = 1;
  }
  let refVoltage = parseFloat(common.refVoltage.value);
  if (filterType != "SINC" && common.pgaGain.value != "Bypass") {
    bypassScaleVal = 1;
  }
  if (filterType == "FIR") {
    digitalScaleVal = 1;
  }
  return (
    (refVoltage * fsrScaleVal * bypassScaleVal * digitalScaleVal) / pgaGain
  );
}

/**
 * Function that needs to be exported for computing dynamic calculation of control values
 * arguments should be in the same order as it will be passed from the GUI
 * controlData   : Entire Object
 * parameters     : Parameters stored in the JSON file
 * common        : Common Controls in parameters JSON file
 * codeToVoltage : Code to Voltage Controls in parameters JSON file
 * voltageToCode : Voltage to Code Controls in parameters JSON file
 */
export function updateControlInfo(
  controlData,
  parameters,
  common,
  codeToVoltage,
  voltageToCode,
) {
  let filterType = common.filter.value;
  switch (filterType) {
    case "FIR":
      parameters.resolution = 32;
      break;
    case "SINC":
      parameters.resolution = 31;
      break;
  }
}

//Helper Functions
function getNumOfCodesValue(parameters) {
  const bitResolution = parseInt(parameters.resolution);
  return 2 ** bitResolution;
}
